from django.contrib import admin

from .models import PasswordHistory, UserPasswordHistoryConfig


class UserPasswordHistoryConfigAdmin(admin.ModelAdmin):

    list_display = ('user', 'date', 'iterations')
    list_filter = ('date', )
    ordering = ('date', )


class PasswordHistoryAdmin(admin.ModelAdmin):

    list_display = ('user_config', 'date', )
    list_filter = ('date', )
    ordering = ('date', )

admin.site.register(UserPasswordHistoryConfig, UserPasswordHistoryConfigAdmin)
admin.site.register(PasswordHistory, PasswordHistoryAdmin)



# Versions:
#
# X.Y.ZAB
# X - Stable major version of the application,
# Y - Minor release with additional features
# Z - Version with bug fixes
# A - Pre-release types:
#      "a" - alpha
#      "b" - betha
#      "c" - release candidate
# B - Pre-release number
#
# X,Y,Z - range <0, inf)
# B - range <1, inf)
#
# Examples:
#  1.1
#  0.3.5b1
#  0.6c3
__version__ = '0.1.0'



from django.contrib.auth.hashers import PBKDF2PasswordHasher


class HistoryHasher(PBKDF2PasswordHasher):
    """
    We need to keep the old password so that when you update django 
    (or configuration change) hashes have not changed. 
    Therefore, special hasher.

    """
    # Experimental value of the of iterations so that the calculation on the
    # average server configuration lasted around one second.
    iterations = 20000 * 10


class HistoryVeryStrongHasher(PBKDF2PasswordHasher):
    """
    We need to keep the old password so that when you update django 
    (or configuration change) hashes have not changed. 
    Therefore, special hasher.

    """
    # Experimental value of the of iterations so that the calculation on the
    # average server configuration lasted around 10 second.
    iterations = 20000 * 101



from django.conf import settings
from django.db import models
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext as _

from django_password_validators.settings import get_password_hasher


class UserPasswordHistoryConfig(models.Model):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        editable=False
    )
    date = models.DateTimeField(
        _('When created salt'),
        auto_now_add=True,
        editable=False
    )
    salt = models.CharField(
        verbose_name=_('Salt for the user'),
        max_length=120,
        editable=False,
    )
    iterations = models.IntegerField(
        _('The number of of iterations for Hasher'),
        default=None,
        editable=False,
        blank=True,
        null=True
    )

    class Meta:
        verbose_name = _('Configuration')
        verbose_name_plural = _('Configurations')
        unique_together = (("user", "iterations",),)
        ordering = ['-user', 'iterations', ]

    def make_password_hash(self, password):
        """
        Generates a password  hash for the given password.

        Args:
            passaword - the password is not encrypted form
        """
        hasher = get_password_hasher()()
        return hasher.encode(password, self.salt, self.iterations)

    def _gen_password_history_salt(self):
        salt_max_length = self._meta.get_field('salt').max_length
        self.salt = get_random_string(length=salt_max_length)

    def save(self, *args, **kwargs):
        # When there is no salt as defined for a given user,
        # then we create the salt.
        if not self.salt:
            self._gen_password_history_salt()
        # We take iterations from the default Hasher
        if not self.iterations:
            self.iterations = get_password_hasher().iterations
        return super(UserPasswordHistoryConfig, self).save(*args, **kwargs)

    def __str__(self):
        return '%s [%d]' % (self.user, self.iterations)


class PasswordHistory(models.Model):
    user_config = models.ForeignKey(
        UserPasswordHistoryConfig,
        editable=False
    )
    password = models.CharField(
        _('Password hash'),
        max_length=256,
        editable=False
    )
    date = models.DateTimeField(
        _('Date'),
        auto_now_add=True,
        editable=False
    )

    class Meta:
        verbose_name = 'Old password'
        verbose_name_plural = 'Password history'
        unique_together = (("user_config", "password",),)
        ordering = ['-user_config', 'password', ]

    def __str__(self):
        return '%s [%s]' % (self.user_config.user, self.date)



from django.conf import settings
from django.utils.module_loading import import_string


def get_password_hasher():
    history_hasher = getattr(
        settings,
        'DPV_DEFAULT_HISTORY_HASHER',
        'django_password_validators.password_history.hashers.HistoryHasher'
    )
    return import_string(history_hasher)



from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from django_password_validators.settings import get_password_hasher
from django_password_validators.password_history.models import (
    PasswordHistory,
    UserPasswordHistoryConfig,
)


class UniquePasswordsValidator(object):
    """
    Validate whether the password was once used by the user.

    The password is only checked for an existing user.
    """

    def validate(self, password, user=None):

        if not user:
            return

        for user_config in UserPasswordHistoryConfig.objects.filter(user=user):
            password_hash = user_config.make_password_hash(password)
            try:
                PasswordHistory.objects.get(
                    user_config=user_config,
                    password=password_hash
                )
                raise ValidationError(
                    _("You can not use a password that is already used in this application."),
                    code='password_used'
                )
            except PasswordHistory.DoesNotExist:
                pass

    def password_changed(self, password, user=None):

        if not user:
            return

        user_config = UserPasswordHistoryConfig.objects.filter(
            user=user,
            iterations=get_password_hasher().iterations
        ).first()

        if not user_config:
            user_config = UserPasswordHistoryConfig()
            user_config.user = user
            user_config.save()

        password_hash = user_config.make_password_hash(password)

        # We are looking hash password in the database
        try:
            PasswordHistory.objects.get(
                user_config=user_config,
                password=password_hash
            )
        except PasswordHistory.DoesNotExist:
            ols_password = PasswordHistory()
            ols_password.user_config = user_config
            ols_password.password = password_hash
            ols_password.save()

    def get_help_text(self):
        return _('Your new password can not be identical to any of the '
                 'previously entered.')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-03-01 11:07
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PasswordHistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(editable=False, max_length=256, verbose_name='Password hash')),
                ('date', models.DateTimeField(auto_now_add=True, verbose_name='Date')),
            ],
            options={
                'ordering': ['-user_config', 'password'],
                'verbose_name': 'Old password',
                'verbose_name_plural': 'Password history',
            },
        ),
        migrations.CreateModel(
            name='UserPasswordHistoryConfig',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateTimeField(auto_now_add=True, verbose_name='When created salt')),
                ('salt', models.CharField(editable=False, max_length=120, verbose_name='Salt for the user')),
                ('iterations', models.IntegerField(blank=True, default=None, editable=False, null=True, verbose_name='The number of of iterations for Hasher')),
                ('user', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-user', 'iterations'],
                'verbose_name': 'Configuration',
                'verbose_name_plural': 'Configurations',
            },
        ),
        migrations.AddField(
            model_name='passwordhistory',
            name='user_config',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to='password_history.UserPasswordHistoryConfig'),
        ),
        migrations.AlterUniqueTogether(
            name='userpasswordhistoryconfig',
            unique_together=set([('user', 'iterations')]),
        ),
        migrations.AlterUniqueTogether(
            name='passwordhistory',
            unique_together=set([('user_config', 'password')]),
        ),
    ]
